//QuicksortPrint.java - Rekursiv sorteringsalgoritme
public class QuicksortPrint {
  public static void main(String[] args) {
    String[] elementer = {"hei", "syk", "lot", "dal", "nes", "buk",  "lam"};
    sorter(elementer);
  }

  static <T extends Comparable<T>> void sorter(T[] alleElementer) {
    skrivSegment("Tabell usortert: ", alleElementer, 0, alleElementer.length-1);
    quicksort(alleElementer, 0, alleElementer.length - 1, 1);
    skrivSegment("Tabell sortert: ", alleElementer, 0, alleElementer.length-1);
  }

  static <T extends Comparable<T>> void quicksort(T[] elementer, int frste, int siste, int niv) {
    // Basistilfelle: 0 eller ett element, ingen sortering ndvendig.
    if (frste >= siste) {
      System.out.println(blanker(niv) + niv + "> basistilfelle");
      return;
    }
    // Generelt tilfelle: Partisjoner segmentet.
    skrivSegment(blanker(niv) + niv + "> segment fr deling: ", elementer, frste, siste);
    int skillePosisjon = partisjoner(elementer, frste, siste, niv);
    System.out.println(blanker(niv) + niv + "> skilleelementet: " + elementer[skillePosisjon]);
    skrivSegment(blanker(niv) + niv + "> segment etter deling: ", elementer, frste, siste);

    // Sorter elementene som var mindre eller lik skilleelementet:
    skrivSegment(blanker(niv) + niv + "> sorter: ", elementer, frste, skillePosisjon-1);
    quicksort(elementer, frste, skillePosisjon-1, niv+1);
    skrivSegment(blanker(niv) + niv + "> sortert: ", elementer, frste, skillePosisjon-1);

    // Sorter elementene som var strre enn skilleelementet:
    skrivSegment(blanker(niv) + niv + "> sorter: ", elementer, skillePosisjon+1, siste);
    quicksort(elementer, skillePosisjon+1, siste, niv+1);
    skrivSegment(blanker(niv) + niv + "> sortert: ", elementer, skillePosisjon+1, siste);
  }

  static <T extends Comparable<T>> int partisjoner(T[] elementer, int frste, int siste, int niv) {
    /* Generelt tilfelle: Del opp elementene i dem som er mindre
     eller lik skilleelementet, og dem som er strre enn skilleelementet. */
    T skilleElement = elementer[frste];
    int sisteLite = frste;
    int sisteUdelte = siste;

    // Utfr s lenge det er flere elementer som ikke har blitt delt.
    while (sisteLite < sisteUdelte) {
      // skrivSegment(blanker(niv) + niv + "> deling: ", elementer, frste, siste);
      int frsteUdelte = sisteLite + 1;
      T testElement = elementer[frsteUdelte];
      boolean strre = testElement.compareTo(skilleElement) > 0;
      if (strre) {
        // Bytt om for  plassere elementet etter de udelte.
        elementer[frsteUdelte] = elementer[sisteUdelte];
        elementer[sisteUdelte] = testElement;
        --sisteUdelte;
      } else {
        // Elementet er p riktig side av de udelte allerede.
        ++sisteLite;
      }
    }

    /* Ingen udelte igjen. Plassr skilleelementet mellom de store og
       sm elementene ved  bytte plass med det siste av de sm. */
    int skillePosisjon = sisteLite;
    elementer[frste] = elementer[skillePosisjon];
    elementer[skillePosisjon] = skilleElement;
    // skrivSegment(blanker(niv) + niv + "> deling: ", elementer, frste, siste);
    return skillePosisjon;
  }

  static <T> void skrivSegment(String ledetekst, T[] tab, int fom, int tom) {
    System.out.print(ledetekst);
    for (int i=fom; i<= tom; i++) System.out.print(tab[i] + " ");
    System.out.println();
  }

  static String blanker(int antall) {
    StringBuffer blanke = new StringBuffer();
    for (int i=0; i< 4*(antall-1); i++)
      blanke.append(' ');
    return blanke.toString();
  }
}